use deadpool_redis::{Config, Connection, Pool};
use lazy_static::lazy_static;
use redis::{Cmd, ExpireOption};

use crate::{config, model::response::{ResErr, ResResult}};

pub struct RedisConnManager {
    pool: Pool
}

impl Default for RedisConnManager {
    fn default() -> Self {
        let redis = &config!().redis;
        let url = format!("redis://:{}@{}/{}", redis.password, redis.url, redis.db);
        // tracing::info!("redis url:   {}", url);
        let cfg = Config::from_url(url);
        let pool = cfg.create_pool(Some(deadpool_redis::Runtime::Tokio1)).expect("获取redis连接失败");
        Self { pool }
    }
}

impl RedisConnManager {
    pub async fn set(&self, key: &str, value: &str) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::set(key, value).exec_async(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis set key:{} value:{} error:{}", key, value, err);
                Err(ResErr::service("redis 执行 set 命令失败"))
            },
        }
    }

    pub async fn setexp(&self, key: &str, value: &str, seconds: u64) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::set_ex(key, value, seconds).exec_async(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis setex key:{} value:{} seconds:{} error:{}", key, value, seconds, err);
                Err(ResErr::service("redis 执行 setex 命令失败"))
            },
        }
    }

    pub async fn get(&self, key: &str) -> ResResult<String> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::get(key).query_async::<String>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis get key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 get 命令失败"))
            },
        }
    }

    pub async fn del(&self, key: &str) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::del(key).query_async::<i64>(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis del key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 del 命令失败"))
            },
        }
    }

    pub async fn hset(&self, key: &str, field: &str, value: &str) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hset(key, field, value).query_async::<i64>(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis hset key:{} field:{} value:{} error:{}", key, field, value, err);
                Err(ResErr::service("redis 执行 hset 命令失败"))
            },
        }
    }

    pub async fn hset_ex(&self, key: &str, field: &str, value: &str, expire: i64) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hset(key, field, value).query_async::<i64>(&mut conn).await {
            Ok(_) => {
                match Cmd::hexpire(key, expire, ExpireOption::NONE, field).query_async::<Vec<i64>>(&mut conn).await {
                    Ok(_) => Ok(()),
                    Err(err) => {
                        tracing::error!("redis hexpire key:{} field:{} value:{} error:{}", key, field, value, err);
                        Err(ResErr::service("redis 执行 hexpire 命令失败"))
                    },
                }
            },
            Err(err) => {
                tracing::error!("redis hset key:{} field:{} value:{} error:{}", key, field, value, err);
                Err(ResErr::service("redis 执行 hset 命令失败"))
            },
        }
    }

    pub async fn hget(&self, key: &str, field: &str) -> ResResult<String> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hget(key, field).query_async::<String>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis hget key:{} field:{} error:{}", key, field, err);
                Err(ResErr::service("redis 执行 hget 命令失败"))
            },
        }
    }

    pub async fn hdel(&self, key: &str, field: &str) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hdel(key, field).query_async::<i64>(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis hdel key:{} field:{} error:{}", key, field, err);
                Err(ResErr::service("redis 执行 hdel 命令失败"))
            },
        }
    }

    pub async fn hgetall(&self, key: &str) -> ResResult<Vec<(String, String)>> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hgetall(key).query_async::<Vec<(String, String)>>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis hgetall key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 hgetall 命令失败"))
            },
        }
    }

    pub async fn hkeys(&self, key: &str) -> ResResult<Vec<String>> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hkeys(key).query_async::<Vec<String>>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis hkeys key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 hkeys 命令失败"))
            },
        }
    }

    pub async fn hvals(&self, key: &str) -> ResResult<Vec<String>> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hvals(key).query_async::<Vec<String>>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis hvals key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 hvals 命令失败"))
            },
        }
    }
    
    pub async fn hlen(&self, key: &str) -> ResResult<i64> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::hlen(key).query_async::<i64>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis hlen key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 hlen 命令失败"))
            },
        }
    }

    pub async fn expire(&self, key: &str, seconds: i64) -> ResResult<()> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::expire(key, seconds).query_async::<i64>(&mut conn).await {
            Ok(_) => Ok(()),
            Err(err) => {
                tracing::error!("redis expire key:{} seconds:{} error:{}", key, seconds, err);
                Err(ResErr::service("redis 执行 expire 命令失败"))
            },
        }
    }

    pub async fn ttl(&self, key: &str) -> ResResult<i64> {
        let mut conn = self.get_conn().await.unwrap();
        match Cmd::ttl(key).query_async::<i64>(&mut conn).await {
            Ok(result) => Ok(result),
            Err(err) => {
                tracing::error!("redis ttl key:{} error:{}", key, err);
                Err(ResErr::service("redis 执行 ttl 命令失败"))
            },
        }
    }

    #[inline]
    async fn get_conn(&self) -> ResResult<Connection> {
        match self.pool.get().await {
            Ok(conn) => Ok(conn),
            Err(err) => {
                tracing::error!("redis get_conn error:{}", err);
                Err(ResErr::service("redis 连接池获取连接失败"))
            },
        }
    }
}

lazy_static! {
    pub static ref REDIS_CACHE: RedisConnManager = RedisConnManager::default();
}